@@ -1,17 +1,11 @@
-# -*- coding: utf-8 -*-
 """ support for providing temporary directories to test functions.  """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import os
 import re
 import tempfile
-import warnings
+from typing import Optional
 
 import attr
 import py
-import six
 
 import pytest
 from .pathlib import ensure_reset_dir
@@ -19,28 +13,30 @@ from .pathlib import LOCK_TIMEOUT
 from .pathlib import make_numbered_dir
 from .pathlib import make_numbered_dir_with_cleanup
 from .pathlib import Path
-from _pytest.monkeypatch import MonkeyPatch
+from _pytest.fixtures import FixtureRequest
 
 
 @attr.s
-class TempPathFactory(object):
+class TempPathFactory:
     """Factory for temporary directories under the common base temp directory.
 
     The base directory can be configured using the ``--basetemp`` option."""
 
     _given_basetemp = attr.ib(
+        type=Path,
         # using os.path.abspath() to get absolute path instead of resolve() as it
         # does not work the same in all platforms (see #4427)
         # Path.absolute() exists, but it is not public (see https://bugs.python.org/issue25012)
+        # Ignore type because of https://github.com/python/mypy/issues/6172.
         converter=attr.converters.optional(
-            lambda p: Path(os.path.abspath(six.text_type(p)))
-        )
+            lambda p: Path(os.path.abspath(str(p)))  # type: ignore
+        ),
     )
     _trace = attr.ib()
-    _basetemp = attr.ib(default=None)
+    _basetemp = attr.ib(type=Optional[Path], default=None)
 
     @classmethod
-    def from_config(cls, config):
+    def from_config(cls, config) -> "TempPathFactory":
         """
         :param config: a pytest configuration
         """
@@ -48,8 +44,30 @@ class TempPathFactory(object):
             given_basetemp=config.option.basetemp, trace=config.trace.get("tmpdir")
         )
 
-    def mktemp(self, basename, numbered=True):
-        """makes a temporary directory managed by the factory"""
+    def _ensure_relative_to_basetemp(self, basename: str):
+        basename = os.path.normpath(basename)
+        if (self.getbasetemp() / basename).resolve().parent != self.getbasetemp():
+            raise ValueError(
+                "{} is not a normalized and relative path".format(basename)
+            )
+        return basename
+
+    def mktemp(self, basename: str, numbered: bool = True) -> Path:
+        """Creates a new temporary directory managed by the factory.
+
+        :param basename:
+            Directory base name, must be a relative path.
+
+        :param numbered:
+            If True, ensure the directory is unique by adding a number
+            prefix greater than any existing one: ``basename="foo"`` and ``numbered=True``
+            means that this function will create directories named ``"foo-0"``,
+            ``"foo-1"``, ``"foo-2"`` and so on.
+
+        :return:
+            The path to the new directory.
+        """
+        basename = self._ensure_relative_to_basetemp(basename)
         if not numbered:
             p = self.getbasetemp().joinpath(basename)
             p.mkdir()
@@ -58,7 +76,7 @@ class TempPathFactory(object):
             self._trace("mktemp", p)
         return p
 
-    def getbasetemp(self):
+    def getbasetemp(self) -> Path:
         """ return base temporary directory. """
         if self._basetemp is not None:
             return self._basetemp
@@ -85,31 +103,17 @@ class TempPathFactory(object):
 
 
 @attr.s
-class TempdirFactory(object):
+class TempdirFactory:
     """
     backward comptibility wrapper that implements
     :class:``py.path.local`` for :class:``TempPathFactory``
     """
 
-    _tmppath_factory = attr.ib()
+    _tmppath_factory = attr.ib(type=TempPathFactory)
 
-    def ensuretemp(self, string, dir=1):
-        """ (deprecated) return temporary directory path with
-            the given string as the trailing part.  It is usually
-            better to use the 'tmpdir' function argument which
-            provides an empty unique-per-test-invocation directory
-            and is guaranteed to be empty.
+    def mktemp(self, basename: str, numbered: bool = True) -> py.path.local:
         """
-        # py.log._apiwarn(">1.1", "use tmpdir function argument")
-        from .deprecated import PYTEST_ENSURETEMP
-
-        warnings.warn(PYTEST_ENSURETEMP, stacklevel=2)
-        return self.getbasetemp().ensure(string, dir=dir)
-
-    def mktemp(self, basename, numbered=True):
-        """Create a subdirectory of the base temporary directory and return it.
-        If ``numbered``, ensure the directory is unique by adding a number
-        prefix greater than any existing one.
+        Same as :meth:`TempPathFactory.mkdir`, but returns a ``py.path.local`` object.
         """
         return py.path.local(self._tmppath_factory.mktemp(basename, numbered).resolve())
 
@@ -118,7 +122,7 @@ class TempdirFactory(object):
         return py.path.local(self._tmppath_factory.getbasetemp().resolve())
 
 
-def get_user():
+def get_user() -> Optional[str]:
     """Return the current user name, or None if getuser() does not work
     in the current environment (see #1010).
     """
@@ -130,37 +134,21 @@ def get_user():
         return None
 
 
-def pytest_configure(config):
-    """Create a TempdirFactory and attach it to the config object.
-
-    This is to comply with existing plugins which expect the handler to be
-    available at pytest_configure time, but ideally should be moved entirely
-    to the tmpdir_factory session fixture.
-    """
-    mp = MonkeyPatch()
-    tmppath_handler = TempPathFactory.from_config(config)
-    t = TempdirFactory(tmppath_handler)
-    config._cleanup.append(mp.undo)
-    mp.setattr(config, "_tmp_path_factory", tmppath_handler, raising=False)
-    mp.setattr(config, "_tmpdirhandler", t, raising=False)
-    mp.setattr(pytest, "ensuretemp", t.ensuretemp, raising=False)
-
-
 @pytest.fixture(scope="session")
-def tmpdir_factory(request):
+def tmpdir_factory(tmp_path_factory) -> TempdirFactory:
     """Return a :class:`_pytest.tmpdir.TempdirFactory` instance for the test session.
     """
-    return request.config._tmpdirhandler
+    return TempdirFactory(tmp_path_factory)
 
 
 @pytest.fixture(scope="session")
-def tmp_path_factory(request):
+def tmp_path_factory(request: FixtureRequest) -> TempPathFactory:
     """Return a :class:`_pytest.tmpdir.TempPathFactory` instance for the test session.
     """
-    return request.config._tmp_path_factory
+    return TempPathFactory.from_config(request.config)
 
 
-def _mk_tmp(request, factory):
+def _mk_tmp(request: FixtureRequest, factory: TempPathFactory) -> Path:
     name = request.node.name
     name = re.sub(r"[\W]", "_", name)
     MAXVAL = 30
@@ -182,7 +170,7 @@ def tmpdir(tmp_path):
 
 
 @pytest.fixture
-def tmp_path(request, tmp_path_factory):
+def tmp_path(request: FixtureRequest, tmp_path_factory: TempPathFactory) -> Path:
     """Return a temporary directory path object
     which is unique to each test function invocation,
     created as a sub directory of the base temporary
