@@ -1,4 +1,5 @@
 import pytest
+from _pytest._io.saferepr import _pformat_dispatch
 from _pytest._io.saferepr import saferepr
 
 
@@ -147,3 +148,9 @@ def test_unicode():
     val = "£€"
     reprval = "'£€'"
     assert saferepr(val) == reprval
+
+
+def test_pformat_dispatch():
+    assert _pformat_dispatch("a") == "'a'"
+    assert _pformat_dispatch("a" * 10, width=5) == "'aaaaaaaaaa'"
+    assert _pformat_dispatch("foo bar", width=5) == "('foo '\n 'bar')"
